# -*- coding: utf8 -*-

from multiprocessing import cpu_count
from multiprocessing.pool import ThreadPool

from noseapp.suite.bases.simple import BaseSuite


class ThreadSuite(BaseSuite):
    """
    Run tests with multiprocessing.pool.ThreadPool
    """

    def _run(self, result, orig):
        size = self.config.options.thread_pool

        if size < 0:
            size = cpu_count() / 2

        pool = ThreadPool(int(round(size)) or 2)

        for test in self._tests:
            pool.apply_async(test, args=(result,))

        pool.close()
        pool.join()



# -*- coding: utf-8 -*-

"""
author: mitsuhiko
source: https://github.com/mitsuhiko/flask.git
"""

import os
import sys


def re_raise(tp, value, tb=None):
    raise tp, value, tb


class ExtensionImporter(object):

    def __init__(self, module_choices, wrapper_module):
        self.module_choices = module_choices
        self.wrapper_module = wrapper_module
        self.prefix = wrapper_module + '.'
        self.prefix_cutoff = wrapper_module.count('.') + 1

    def __eq__(self, other):
        return self.__class__.__module__ == other.__class__.__module__ and \
               self.__class__.__name__ == other.__class__.__name__ and \
               self.wrapper_module == other.wrapper_module and \
               self.module_choices == other.module_choices

    def __ne__(self, other):
        return not self.__eq__(other)

    def install(self):
        sys.meta_path[:] = [x for x in sys.meta_path if self != x] + [self]

    def find_module(self, fullname, path=None):
        if fullname.startswith(self.prefix):
            return self

    def load_module(self, fullname):
        if fullname in sys.modules:
            return sys.modules[fullname]
        modname = fullname.split('.', self.prefix_cutoff)[self.prefix_cutoff]
        for path in self.module_choices:
            realname = path % modname
            try:
                __import__(realname)
            except ImportError:
                exc_type, exc_value, tb = sys.exc_info()
                sys.modules.pop(fullname, None)

                if self.is_important_traceback(realname, tb):
                    re_raise(exc_type, exc_value, tb.tb_next)
                continue
            module = sys.modules[fullname] = sys.modules[realname]
            if '.' not in modname:
                setattr(sys.modules[self.wrapper_module], modname, module)
            return module
        raise ImportError('No module named %s' % fullname)

    def is_important_traceback(self, important_module, tb):
        while tb is not None:
            if self.is_important_frame(important_module, tb):
                return True
            tb = tb.tb_next
        return False

    def is_important_frame(self, important_module, tb):
        g = tb.tb_frame.f_globals
        if '__name__' not in g:
            return False

        module_name = g['__name__']

        if module_name == important_module:
            return True

        filename = os.path.abspath(tb.tb_frame.f_code.co_filename)
        test_string = os.path.sep + important_module.replace('.', os.path.sep)
        return test_string + '.py' in filename or \
               test_string + os.path.sep + '__init__.py' in filename



# -*- coding: utf8 -*-

from unittest import TestCase


class TestClearExtensions(TestCase):
    """
    Set extension, check get extension, call clear, check raise
    """

    def runTest(self):
        from noseapp.core import extensions

        extensions.set('test', dict(test='test'))
        self.assertEqual(extensions.get('test').get('test'), 'test')

        extensions.clear()
        self.assertRaises(extensions.ExtensionNotFound, extensions.get, 'test')



# -*- coding: utf-8 -*-

"""
Realisation from flask lib
"""


def setup():
    from ..exthook import ExtensionImporter
    importer = ExtensionImporter(['noseapp_%s', 'noseapp.%s'], __name__)
    importer.install()


setup()
del setup



# -*- coding: utf-8 -*-

"""
Auto load suites for register in noseapp.NoseApp
"""

import os
from importlib import import_module

from noseapp.suite import Suite


class LoadSuitesError(BaseException):
    pass


def is_exist(path):
    """
    Check exist path
    """
    if not os.path.exists(path):
        raise LoadSuitesError('Dir "{}" does not exist'.format(path))


def is_package(path):
    """
    May be path is python package?
    """
    if not os.path.isfile(os.path.join(path, '__init__.py')):
        raise LoadSuitesError('"{}" is not python package'.format(path))


def load_from_dir(path, import_base=None):
    """
    Load suites from dir

    :type path: str
    :param import_base: base import path
    :type import_base: str
    """
    if import_base:
        is_package(path)

    suites = []

    py_files = filter(
        lambda f: f.endswith('.py') and not f.startswith('_'),
        os.listdir(path),
    )
    modules = (m.rstrip('.py') for m in py_files)

    for module in modules:

        if import_base:
            module = '{}.{}'.format(import_base, module)

        module = import_module(module)

        module_suites = (
            getattr(module, atr)
            for atr in dir(module)
            if isinstance(
                getattr(module, atr, None), Suite,
            )
        )

        map(suites.append, module_suites)

    return suites


def load_suites_from_path(path, import_base=None):
    """
    Recursive load suites from path

    :param path: path to dir
    :type path: str
    :param import_base: base import path
    :type import_base: str
    """
    is_exist(path)

    suites = []

    copy_import_base = import_base

    map(suites.append, load_from_dir(path, import_base=import_base))

    for root, dirs, files in os.walk(path):

        for d in dirs:
            dir_abs_path = os.path.join(root, d)

            if import_base is None:
                _import_base = d
            else:
                _import_base = '{}.{}'.format(import_base, d)

            map(
                suites.append,
                load_suites_from_path(
                    dir_abs_path,
                    import_base=_import_base,
                ),
            )

            import_base = copy_import_base

    return suites



# -*- coding: utf-8 -*-

from functools import wraps


def flows(*flows):
    """
    Decorator for parametrize test method
    """
    def wrapper(f):
        @wraps(f)
        def wrapped(self, *args, **kwargs):
            for flow in flows:
                f(self, flow, *args, **kwargs)
            return None
        return wrapped
    return wrapper




import os

from noseapp.plugins.base import AppPlugin


class AppConfigurePlugin(AppPlugin):

    name = 'app-configure'

    def options(self, parser, env=os.environ):
        super(AppConfigurePlugin, self).options(parser, env=env)

        parser.add_option(
            '--app-processes',
            dest='app_processes',
            default=0,
            type=int,
            help='Spread test run among this many processes.'
        )
        parser.add_option(
            '--gevent',
            dest='gevent_pool',
            default=0,
            type=int,
            help='Spread test run among this many gevent greanlets.'
        )
        parser.add_option(
            '--greenlets',
            dest='gevent_greanlets',
            default=0,
            type=int,
            help='Spread greenlets inside suite greanlet.'
        )
        parser.add_option(
            '--threads',
            dest='thread_pool',
            default=0,
            type=int,
            help='Spread threads inside process.'
        )
        parser.add_option(
            '--ls',
            dest='ls',
            action='store_true',
            default=False,
            help='Show suites tree.'
        )
        parser.add_option(
            '--doc',
            dest='doc',
            action='store_true',
            default=False,
            help='Show docstring in suite info.'
        )
        parser.add_option(
            '-t', '--test',
            dest='run_test',
            default='',
            type=str,
            help='test case or suite name for running'
        )



# coding: utf-8


class ModifyDict(dict):
    """
    Verb in class name as instruction to usage

    Example:
        from noseapp.datastructures import ModifyDict as Context
    """

    def __getattr__(self, item):
        try:
            return self[item]
        except KeyError:
            raise AttributeError(
                '"{}" does not have "{}" attribute.'.format(self.__class__.__name__, item),
            )

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, item):
        try:
            del self[item]
        except KeyError:
            raise AttributeError(
                '"{}" does not have "{}" attribute.'.format(self.__class__.__name__, item),
            )



# -*- coding: utf-8 -*-

"""
Tmp storage for extensions
"""

from copy import deepcopy

from noseapp.datastructures import ModifyDict as Transport


_EXTENSIONS = {}


class ExtensionNotFound(BaseException):
    pass


class ExtensionNotRequired(BaseException):
    pass


def get(name, require=None):
    """
    Get extension by name

    :param name: extension name
    :type name: basestring
    :param require: allow extensions list
    :type require: list or tuple
    """
    if require is not None and name not in require:
        raise ExtensionNotRequired(name)

    try:
        ext = _EXTENSIONS[name]

        if ext.__class__ is Transport:
            return ext.cls(*ext.args, **ext.kwargs)

        return deepcopy(ext)
    except KeyError:
        raise ExtensionNotFound(name)


def set(name, ext, in_context=False, args=None, kwargs=None):
    """
    Register extension in tmp storage

    :param name: extension name
    :type name: basestring
    :param ext: any objects
    :param in_context: if True, create instance object with call get function
    :type in_context: bool
    """
    if in_context:
        _EXTENSIONS[name] = Transport(
            cls=ext, args=args or tuple(), kwargs=kwargs or dict(),
        )
    else:
        _EXTENSIONS[name] = ext

    return ext


def clear():
    """
    Clear tmp
    """
    global _EXTENSIONS
    _EXTENSIONS = {}



# -*- coding: utf-8 -*-

from unittest import TestCase

from nose.suite import ContextSuite

from noseapp.suite import Suite
from noseapp.core import TestProgram
from noseapp.core import CollectSuite
from noseapp import NoseApp as _NoseApp
from noseapp.suite import TestCaseMediator
from noseapp import TestCase as NoseAppTestCase
from noseapp.plugins.configure import AppConfigurePlugin


class NoseApp(_NoseApp):

    def initialize(self):
        pass


class FakeTestCase(NoseAppTestCase):

    def test(self):
        pass


class TestSuiteClass(TestCase):
    """
    Test case for noseapp.suite.base.Suite
    """

    def test_default_mediator_class(self):
        self.assertEqual(Suite.mediator_class, TestCaseMediator)

    def test_set_require(self):
        require = ['one', 'two']
        suite = Suite(__name__, require=require)

        self.assertEqual(suite.require, require)

    def test_default_require(self):

        class TestSuite(Suite):

            DEFAULT_REQUIRE = ['one']

        suite = TestSuite(__name__, require=['two'])

        self.assertEqual(suite.require, ['one', 'two'])

    def test_set_name(self):
        suite = Suite(__name__)

        self.assertEqual(suite.name, __name__)

    def test_register_test_case(self):
        suite = Suite(__name__)
        suite.register(NoseAppTestCase)

        self.assertEqual(len(suite.get_map()), 1)

    def test_build_suite(self):
        from nose.case import Test
        import noseapp.suite.mediator
        from noseapp.suite.bases.simple import BaseSuite

        mock_get_suite_class = lambda o: BaseSuite
        noseapp.suite.mediator.get_suite_class = mock_get_suite_class

        suite = Suite(__name__)
        suite.register(FakeTestCase)

        program = TestProgram(NoseApp(), ['test'])
        test = suite(program.config, program.testLoader)

        self.assertIsInstance(test, BaseSuite)

        test = [t for t in test]

        self.assertEqual(len(test), 1)
        self.assertIsInstance(test[0], Test)


class TestMediatorClass(TestCase):
    """
    Test case for noseapp.suite.mediator.TestCaseMediator
    """

    def test_set_require(self):
        require = ['one', 'two']
        instance = TestCaseMediator(require)

        self.assertEqual(instance._require, require)
        self.assertEqual(instance._test_cases, [])

    def test_add_test_case(self):
        mediator = TestCaseMediator(None)
        mediator.add_test_case(NoseAppTestCase)

        self.assertIn(NoseAppTestCase, mediator.test_cases)

    def test_create_map(self):
        mediator = TestCaseMediator(None)
        mediator.add_test_case(FakeTestCase)

        mp = mediator.create_map()

        self.assertIn('FakeTestCase', mp)
        self.assertEqual(mp['FakeTestCase']['cls'], FakeTestCase)
        self.assertIn('test', mp['FakeTestCase']['tests'])
        self.assertEqual(mp['FakeTestCase']['tests']['test'], FakeTestCase.test)


class TestCollectorClass(TestCase):
    """
    Test noseapp.suite.collector.CollectSuites class
    TODO: unification algorithm
    """

    def test_collect_suites(self):
        argv = ['test', '--with-app-configure']

        program = TestProgram(
            NoseApp(),
            argv,
            plugins=[AppConfigurePlugin()],
        )
        suites = [Suite(__name__) for _ in xrange(10)]

        for suite in suites:
            suite.register(NoseAppTestCase)

        collector = CollectSuite(
            argv,
            suites,
            program.testLoader,
            program.config,
        )
        result = collector.make_result()

        self.assertIsInstance(result, ContextSuite)

        result = [s for s in result]

        self.assertEqual(len(result), 10)

        for suite in result:
            self.assertIsInstance(suite, ContextSuite)

    def test_collect_one_suite(self):
        expect_suite_name = 'test.test_suite_name'
        expect_suite = Suite(expect_suite_name)
        argv = ['test', expect_suite_name, '--with-app-configure']

        suites = [Suite(__name__) for _ in xrange(9)] + [expect_suite]

        program = TestProgram(
            NoseApp(),
            argv,
            plugins=[AppConfigurePlugin()],
        )

        collector = CollectSuite(
            argv,
            suites,
            program.testLoader,
            program.config,
        )
        result = collector.make_result()

        self.assertIsInstance(result, ContextSuite)

        result = [s for s in result]

        self.assertEqual(len(result), 1)

        for suite in result:
            self.assertIsInstance(suite, ContextSuite)

    def test_collect_one_case(self):
        expect_case_name = FakeTestCase.__name__
        expect_suite_name = 'test_suite_name'

        argv = [
            'test',
            '{}:{}'.format(expect_suite_name, expect_case_name),
            '--with-app-configure'
        ]

        expect_suite = Suite(expect_suite_name)
        expect_suite.register(FakeTestCase)
        expect_suite.register(NoseAppTestCase)

        suites = [Suite(__name__) for _ in xrange(9)] + [expect_suite]

        program = TestProgram(
            NoseApp(),
            argv,
            plugins=[AppConfigurePlugin()],
        )

        collector = CollectSuite(
            argv,
            suites,
            program.testLoader,
            program.config,
        )
        result = collector.make_result()

        self.assertIsInstance(result, ContextSuite)

        result = [s for s in result]

        self.assertEqual(len(result), 1)

        for suite in result:
            self.assertIsInstance(suite, ContextSuite)

            for test in suite:
                self.assertIsInstance(test.test, FakeTestCase)

    def test_collect_one_case_method(self):

        class NewFakeTestCase(FakeTestCase):

            def test_(self):
                pass

        expect_case_name = NewFakeTestCase.__name__
        expect_suite_name = 'test_suite_name.test'

        argv = [
            'test',
            '{}:{}'.format(expect_suite_name, expect_case_name),
            '--with-app-configure'
        ]

        expect_suite = Suite(expect_suite_name)
        expect_suite.register(NewFakeTestCase)
        expect_suite.register(NoseAppTestCase)

        suites = [Suite(__name__) for _ in xrange(9)] + [expect_suite]

        program = TestProgram(
            NoseApp(),
            argv,
            plugins=[AppConfigurePlugin()],
        )

        collector = CollectSuite(
            argv,
            suites,
            program.testLoader,
            program.config,
        )
        result = collector.make_result()

        self.assertIsInstance(result, ContextSuite)

        result = [s for s in result]

        self.assertEqual(len(result), 1)

        for suite in result:
            self.assertIsInstance(suite, ContextSuite)

            for test in suite:
                self.assertIsInstance(test.test, FakeTestCase)



# -*- coding: utf8 -*-

import sys
from warnings import warn


def _print_line(string, spaces=0):
    sys.stdout.write('| {}{}\n'.format(_get_spaces(spaces), string))


def _print_total_info(suites, cases, methods):
    info = 'Total: suites: {}. cases: {}. methods: {}.\n'.format(
        suites, cases, methods,
    )

    separate = []

    for i in xrange(len(info) - 1):
        separate.append('-')

    sys.stdout.write('\n{}\n'.format(''.join(separate)))
    sys.stdout.write(info)


def _get_spaces(num):
    s = []

    for i in xrange(num):
        s.append(' ')

    return ''.join(s)


def _li(string):
    return '\_ * {}'.format(string)


def _doc(string, is_end):
    return '| - {}{}'.format(string, '...' if is_end else '')


def tree(suites, show_docs=True, doc_lines=1):
    """
    Print suites tree from application

    :param suites: suite list from app
    :param show_docs: show docstring flag
    :param doc_lines: number of docstring lines
    """
    case_counter = 0
    suite_counter = 0
    method_counter = 0
    suite_names = set()

    # Suites
    for suite in suites:
        suite_counter += 1
        suite_name = suite.name

        if suite_name in suite_names:
            warn(
                'Duplicate suite name "{}"! Please rename your suite!'.format(
                    suite_name,
                ),
            )
        suite_names.add(suite_name)

        _print_line('* {}'.format(suite_name))

        mp = suite.get_map()

        # Cases
        for cls_name in mp:
            case_counter += 1

            _print_line(
                _li(
                    '{case} ({suite}:{case})'.format(
                        case=cls_name,
                        suite=suite_name,
                    ),
                ),
                spaces=2,
            )

            if show_docs:
                cls_doc = mp[cls_name]['cls'].__doc__

                if cls_doc:
                    line_counter = 0

                    for line in cls_doc.splitlines():
                        line = line.strip()

                        if line:
                            is_end = line_counter == doc_lines - 1

                            _print_line(_doc(line, is_end), spaces=4)

                            line_counter += 1

                            if is_end:
                                break

            # if test method only one, no print info
            if len(mp[cls_name]['tests']) == 1:
                method_counter += 1
                continue

            # Tests
            for test_method in mp[cls_name]['tests']:
                method_counter += 1

                _print_line(
                    _li(
                        '{test} ({suite}:{case}.{test})'.format(
                            test=test_method,
                            case=cls_name,
                            suite=suite_name,
                        ),
                    ),
                    spaces=4,
                )

                if show_docs:
                    method_doc = mp[cls_name]['tests'][test_method].__doc__

                    if method_doc:
                        line_counter = 0

                        for line in method_doc.splitlines():
                            line = line.strip()

                            if line:
                                is_end = line_counter == doc_lines - 1

                                _print_line(_doc(line, is_end), spaces=6)

                                line_counter += 1

                                if is_end:
                                    break

    _print_total_info(suite_counter, case_counter, method_counter)

    sys.exit(0)



# -*- coding: utf8 -*-

from multiprocessing import cpu_count

from gevent.pool import Pool

from noseapp.suite.bases.simple import BaseSuite


class GeventSuite(BaseSuite):
    """
    Run tests with gevent pool
    """

    def _run(self, result, orig):
        size = self.config.options.gevent_greanlets

        if size < 0:
            size = cpu_count() - 1

        pool = Pool(int(round(size)) or 2)

        for test in self._tests:
            pool.spawn(test, orig)

        pool.join()



# -*- coding: utf-8 -*-

from unittest import TestCase as BaseTestCase

from noseapp.core import extensions


class TestCase(BaseTestCase):
    """
    Base case class
    """

    @classmethod
    def with_require(cls, require=None):
        """
        Init require extensions

        :param require: extensions list
        :type require: list or tuple
        """
        if require and hasattr(require, '__iter__'):
            for ext_name in require:
                setattr(cls, ext_name, extensions.get(ext_name, require))

        return cls



# -*- coding: utf-8 -*-

from nose.plugins import Plugin as BasePlugin


class AppPlugin(BasePlugin):
    """
    Base application plugin
    """

    def __call__(self, app):
        """
        Push app instance

        :type app: noseapp.NoseApp
        """
        self.app = app
        return self



# -*- coding: utf8 -*-

from unittest import TestCase


class TestFlowsDecorator(TestCase):
    """
    Test decorator for parametrize test method
    """

    @staticmethod
    def create_fake_case():
        from noseapp.decorators import flows

        class FakeCase(object):
            counter = 0

            @flows(1, 2, 3)
            def run(self, i):
                self.counter += i

        return FakeCase()

    def runTest(self):
        case = self.create_fake_case()
        case.run()

        self.assertEqual(case.counter, 6)



# -*- coding: utf8 -*-

from multiprocessing import cpu_count

from gevent.pool import Pool

from noseapp.runner.base import measure_time
from noseapp.runner.base import BaseTestRunner


class GeventTestRunner(BaseTestRunner):
    """
    Run suites with gevent
    """

    def run(self, suites):
        wrapper = self.config.plugins.prepareTest(suites)
        if wrapper is not None:
            suites = wrapper

        wrapped = self.config.plugins.setOutputStream(self.stream)
        if wrapped is not None:
            self.stream = wrapped

        result = self._makeResult()

        size = self.config.options.gevent_pool
        if size < 0:
            size = cpu_count()

        pool = Pool(size)

        with measure_time(result):
            for suite in suites:
                pool.spawn(suite, result)
            pool.join()

        self.config.plugins.finalize(result)
        return result



# -*- coding: utf-8 -*-

import os
from unittest import TestCase

from noseapp import Suite
from noseapp.core import extensions
from noseapp.app import NoseApp as _NoseApp


class NoseApp(_NoseApp):

    def initialize(self):
        pass


class TestDefaultClasses(TestCase):
    """
    Test default classes in NoseApp class property
    """

    def runTest(self):
        from noseapp.core import TestProgram
        from noseapp.app.config import Config

        self.assertEqual(NoseApp.config_class, Config)
        self.assertEqual(NoseApp.program_class, TestProgram)


class TestInitializeCallback(TestCase):
    """
    Test call initialize callback
    """

    def create_fake_app(self):

        class FakeApp(NoseApp):

            def initialize(self):
                self.initialize_data = 1

        return FakeApp()

    def runTest(self):
        fake_app = self.create_fake_app()
        self.assertEqual(fake_app.initialize_data, 1)


class TestBeforeAfterCallback(TestCase):
    """
    Test call before and after callbacks
    """

    def create_fake_app(self):

        class FakeApp(NoseApp):

            def before(self):
                self.before_data = 1

            def after(self):
                self.after_data = 2

        return FakeApp()

    @staticmethod
    def mock_test_runner():
        import noseapp.core.program

        def get_test_runner_class(options):

            class FakeResult(object):

                def wasSuccessful(self):
                    return True

            class FakeTestRunner(object):

                def __init__(self, *args, **kwargs):
                    pass

                def run(self, suites):
                    return FakeResult()

            return FakeTestRunner

        noseapp.core.program.get_test_runner_class = get_test_runner_class

    def runTest(self):
        self.mock_test_runner()

        fake_app = self.create_fake_app()

        try:
            fake_app.run()
        except SystemExit as e:
            self.assertEqual(e.code, 0)

        self.assertEqual(fake_app.before_data, 1)
        self.assertEqual(fake_app.after_data, 2)


class TestRegisterOneSuite(TestCase):
    """
    Register one suite in application
    """

    def runTest(self):
        suite = Suite(__name__)
        app = NoseApp()

        app.register_suite(suite)

        self.assertEqual(len(app.suites), 1)
        self.assertIsInstance(app.suites[0], Suite)


class TestRegisterListSuites(TestCase):
    """
    Register list suites in application
    """

    def runTest(self):
        suites = [Suite(__name__) for _ in xrange(10)]
        app = NoseApp()

        app.register_suites(suites)

        self.assertEqual(len(app.suites), 10)

        for suite in app.suites:
            self.assertIsInstance(suite, Suite)


class TestLoadSuites(TestCase):
    """
    Test auto load suites
    """

    def runTest(self):
        dir_path = os.path.abspath(
            os.path.join(
                os.path.dirname(__file__),
                'fake_suites',
            ),
        )
        app = NoseApp()

        app.load_suites(dir_path)

        self.assertEqual(len(app.suites), 4)

        for suite in app.suites:
            self.assertIsInstance(suite, Suite)


class TestSharedExtension(TestCase):
    """
    Test shared extension in application
    """

    @staticmethod
    def get_fake_ex():

        class ExampleExtension(object):
            name = 'example'

            def __init__(self, *args, **kwargs):
                self.args = args
                self.kwargs = kwargs

        return ExampleExtension

    def test_shared_by_name(self):
        ex = self.get_fake_ex()
        app = NoseApp()

        app.shared_extension(name='test', cls=ex)

        self.assertIsInstance(extensions.get('test'), ex)

    def test_shared_by_class_name(self):
        ex = self.get_fake_ex()
        app = NoseApp()

        app.shared_extension(cls=ex)

        self.assertIsInstance(extensions.get(ex.name), ex)

    def test_shared_with_args_kwargs(self):
        ex = self.get_fake_ex()
        app = NoseApp()

        app.shared_extension(cls=ex, args=(1, 2), kwargs={'test': 'test'})

        get_ex = extensions.get(ex.name)

        self.assertIsInstance(get_ex, ex)
        self.assertEqual(len(get_ex.args), 2)
        self.assertEqual(get_ex.kwargs.get('test'), 'test')

    def test_shared_data(self):
        data = {'test': 'test'}

        app = NoseApp()
        app.shared_data('test_data', data)

        get_data = extensions.get('test_data')

        self.assertEqual(get_data.get('test'), 'test')

        get_data['test'] = 1

        self.assertEqual(data.get('test'), 'test')

    def test_raises(self):
        from noseapp.core.extensions import ExtensionNotFound
        from noseapp.core.extensions import ExtensionNotRequired

        ex = self.get_fake_ex()
        app = NoseApp()

        delattr(ex, 'name')

        self.assertRaises(AttributeError, app.shared_extension, cls=ex)
        self.assertRaises(ExtensionNotFound, extensions.get, 'no_ex')
        self.assertRaises(ExtensionNotRequired, extensions.get, 'test', require=['test2'])


class TestInitConfig(TestCase):
    """
    Test init app configuration
    """

    def test_config_from_module(self):
        from noseapp.core.program import TestProgram

        app = NoseApp()
        app.config.from_module('noseapp.core.program')

        self.assertEqual(app.config.TestProgram, TestProgram)

    def test_config_from_py_file(self):
        path = os.path.abspath(
            os.path.join(
              os.path.dirname(__file__),
              'fake_suites',
              'one.py',
            ),
        )
        app = NoseApp()
        app.config.from_py_file(path)

        self.assertIsInstance(app.config.suite1, Suite)

    def test_init_nose_config(self):

        class FakeNoseConfig(object):

            class options:
                test = 'test'

        app = NoseApp()
        app.config.init_nose_config(FakeNoseConfig())

        self.assertEqual(app.config.nose.test, 'test')



# -*- coding: utf-8 -*-

import os
import imp
import logging
from importlib import import_module

from noseapp.datastructures import ModifyDict as BaseConfig


logger = logging.getLogger(__name__)


def _load(obj):
    for atr in dir(obj):
        if not atr.startswith('_'):
            yield (atr, getattr(obj, atr))


class ConfigError(BaseException):
    pass


class Config(BaseConfig):
    """
    App config storage
    """

    def init_nose_config(self, nose_config):
        """
        Merge options from nose argument parser
        """
        logger.debug('Initialize nose config options')

        self['nose'] = BaseConfig(
            _load(nose_config.options),
        )

    def from_module(self, module):
        """
        Init configuration from python module

        :param module: import path
        :type module: str
        """
        logger.debug('Init config from module "%s"', module)

        try:
            obj = import_module(module)
        except ImportError:
            raise ImportError('Config {} not found'.format(module))

        self.update(_load(obj))

        return self

    def from_py_file(self, file_path):
        """
        Init configuration from py file

        :param file_path: absolute file path
        :type file_path: str
        """
        logger.debug('Init config from py file "%s"', file_path)

        if not os.path.isfile(file_path):
            raise ConfigError('config file does not exist "{}"'.format(file_path))

        elif not file_path.endswith('.py'):
            raise ConfigError('config file is not python file')

        module = imp.new_module(file_path.rstrip('.py'))
        module.__file__ = file_path

        try:
            execfile(file_path, module.__dict__)
        except IOError as e:
            e.strerror = 'Unable to load file "{}"'.format(e.strerror)
            raise

        self.update(_load(module))

        return self



# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages


__version__ = '1.0.5'


if __name__ == '__main__':
    setup(
        name='noseapp',
        version=__version__,
        url='https://github.com/trifonovmixail/noseapp',
        packages=find_packages(),
        author='Mikhail Trifonov',
        author_email='mikhail.trifonov@corp.mail.ru',
        license='GNU LGPL',
        description='Framework for test development',
        keywords='test unittest framework nose application',
        long_description=open('README.rst').read(),
        include_package_data=True,
        zip_safe=False,
        platforms='any',
        install_requires=[
            'nose==1.3.4',
        ],
        entry_points={
            'console_scripts':
                [
                    'noseapp-manage = noseapp.manage:run',
                ],
        },
        test_suite='tests',
        classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2.7',
            'Topic :: Software Development :: Testing',
        ],
    )



# -*- coding: utf-8 -*-

import os
import sys
import logging
from argparse import ArgumentError

from noseapp.core import extensions
from noseapp.core import TestProgram
from noseapp.core import load_suites_from_path
from noseapp.app.config import Config as AppConfig
from noseapp.plugins.configure import AppConfigurePlugin


logger = logging.getLogger(__name__)


DEFAULT_PLUGINS = [
    AppConfigurePlugin(),
]

DEFAULT_ARGV = sys.argv + [
    '--with-app-configure',
]


def prepare_argv(argv):
    """
    :type argv: list
    """
    argv = DEFAULT_ARGV + (argv or [])

    for arg in argv:
        if '--processes' in arg:
            raise ArgumentError(None, 'Option "--processes" is deprecated')
        if arg == '--ls':
            os.environ.setdefault('NOSE_NOCAPTURE', '1')

    return argv


class NoseApp(object):
    """
    Base app class
    """

    config_class = AppConfig
    program_class = TestProgram

    def __init__(self, config=None, plugins=None, argv=None):
        """
        :type config: str
        :type plugins: list
        :type argv: list
        """
        self.config = self.config_class()

        if config:
            if os.path.isfile(config):
                self.config.from_py_file(config)
            else:
                self.config.from_module(config)

        self._suites = []

        self.__test_program = self.program_class(
            app=self,
            exit=True,
            argv=prepare_argv(argv),
            addplugins=[p(self) for p in (DEFAULT_PLUGINS + (plugins or []))],
        )

    def initialize(self):
        """
        Callback for initialize application
        """
        raise NotImplementedError

    def before(self):
        """
        Call before run suites
        """
        pass

    def after(self):
        """
        Call after run suites
        """
        pass

    @staticmethod
    def shared_extension(name=None, cls=None, args=None, kwargs=None):
        """
        Shared extension to TestCase classes. Use require param in noseapp.Suite class.

        :param name: extension name. this is property name in case class.
        :type name: str
        :param cls: extension class
        :param args, kwargs: class init arguments
        :type args: tuple
        :type kwargs: dict
        """
        if cls is None:
            raise ValueError('cls param is required')

        if name is None and not hasattr(cls, 'name'):
            raise AttributeError(
                '"{}" does not have "name" attribute'.format(cls.__name__),
            )
        elif name is None:
            name = cls.name

        logger.debug('Shared extension "%s"', name)

        return extensions.set(name, cls, in_context=True, args=args, kwargs=kwargs)

    @staticmethod
    def shared_data(name, data):
        """
        Shared extension to TestCase classes.
        Data will be copied during installation.

        :param name: this is property name in case class
        :type name: str
        :param data: any object
        """
        logger.debug('Shared data "%s"', name)

        return extensions.set(name, data, in_context=False)

    @property
    def suites(self):
        return self._suites

    def register_suite(self, suite):
        """
        Add suite in application

        :type suite: app.suite.Suite
        """
        logger.debug('Register suite "%s"', suite.name)

        self._suites.append(suite)

    def register_suites(self, suite):
        """
        App suite list in application

        :type suite: list, tuple
        """
        for s in suite:
            self.register_suite(s)

    def load_suites(self, path):
        """
        Auto load suites. Path can be package or simple dir.

        :param path: dir path
        :type path: str
        """
        sys.path.append(path)
        suites = load_suites_from_path(path)
        self.register_suites(suites)

    def run(self):
        """
        Perform run suites
        """
        logger.debug('Run application')

        self.__test_program.perform()

    def __repr__(self):
        return '<NoseApp: {}>'.format(self.__class__.__name__)
